#!/usr/bin/perl

# 
# Draft script to package VNX scenarios
#
# Author: David Fernández (david@dit.upm.es)
#
# This file is part of the Virtual Networks over LinuX (VNX) Project distribution. 
# (www: http://www.dit.upm.es/vnx - e-mail: vnx@dit.upm.es) 
# 
# Departamento de Ingenieria de Sistemas Telematicos (DIT)
# Universidad Politecnica de Madrid
# SPAIN
#

use strict;
use warnings;
use Getopt::Long;
use File::Basename;
use Cwd qw(abs_path getcwd);

use VNX::Globals;



# Log levels
use constant N   => 0;
use constant V   => 1;
use constant VV  => 2;
use constant VVV => 3;
use constant ERR => 4;

my $usage = <<EOF;
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
Virtual Networks over LinuX (VNX) -- http://www.dit.upm.es/vnx - vnx\@dit.upm.es          
~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

vnx_pack_scenario: simple script to package VNX scenarios in a tgz file 

Usage:  vnx_pack_scenario -f <scenario_dir> [-v <version>] [-r]
              -f -> directory where the scenario is
              -v -> set version number in tgz name
              -r -> include root filesystem
EOF

our %opts = ();     # Command line options hash

Getopt::Long::Configure ( qw{no_auto_abbrev no_ignore_case } ); # case sensitive single-character options
GetOptions (\%opts,
                'pack=s', 'unpack=s', 'include-rootfs|r', 'pack-external-rootfs', 'pack-status', 'pack-version=s'
    ) or vnx_die("Incorrect usage. Type 'vnx -h' for help"); 
                
#                'start', 'create|t', 'shutdown|d', 'destroy|P', 'modify|m=s', 'scenario|s=s', 
#                'save', 'restore', 'suspend', 'resume', 'reboot', 'reset', 'recreate', 'execute|x=s', 'exe-cli=s{1,}' => \@exe_cli, 
#                'show-map:s', 'show-status', 'console:s', 'console-info', 'exe-info', 'clean-host',
#                'create-rootfs=s', 'modify-rootfs=s', 'install-media=s', 'update-aced:s', 'mem=s', 'yes|y',
#                'rootfs-type=s', 'help|h', 'v', 'vv', 'vvv', 'version|V', 'download-rootfs',
#                'f=s', 'c=s', 'T=s', 'config|C=s', 'M=s', 'i', 'g',
#                'user|u:s', '4', '6', 'D', 'no-console|n', 'intervm-delay=s',
#                'e=s', 'w=s', 'F', 'B', 'o=s', 'Z', 'b', 'arch=s', 'vcpu=s', 'kill|k', 'video=s'


my $input_file;
my $xml_dir;
my $scen_bname;
my $pack_version;
my $inc_rootfs;
my $pack_tgz_name;
my $tar_opts = '';
my $tmpfile;


if ($opts{'pack'}) {

    $input_file = $opts{'pack'};
	if (! -f $input_file ) {
	    vnx_die ("file $input_file is not valid (perhaps does not exists).");
	}
	print abs_path($input_file) . "\n";
	$xml_dir = dirname abs_path($input_file);
	$scen_bname = basename $xml_dir;
	wlog (V, "input_file=$input_file", '');
	wlog (V, "xml_dir=$xml_dir", '');
	wlog (V, "scen_bname=$scen_bname", '');

	if ($opts{'pack-version'}) {
	    $pack_version = $opts{'pack-version'};
	    $pack_tgz_name = "${scen_bname}-v${pack_version}";
	} else {
	    $pack_tgz_name = ${scen_bname};
	}
	wlog (V, "pack_tgz_name=$pack_tgz_name");
	
	if ($opts{'include-rootfs'}) {
	    $inc_rootfs = 'yes';
	}   

	# Save current dir
	my $cdir=`pwd`; chomp($cdir);

	# Change dir to scenario upper directory
	chdir "${xml_dir}/..";

	wlog (N, "-- Packaging scenario $scen_bname", '');
	
	my $content="$scen_bname/*.xml $scen_bname/*.cvnx $scen_bname/conf $scen_bname/filesystems/create* $scen_bname/filesystems/rootfs*";
	wlog (N, "--   Scenario file content: $content", '');
	
	if ( $inc_rootfs ) {
	    my $rootfs=`readlink $scen_bname/filesystems/rootfs*`; 
	    chomp ($rootfs);
	    wlog (N, "--   Including rootfs: $rootfs", '');
	    $content="$content $scen_bname/filesystems/$rootfs";
	
	    # Exclude sockets to avoid errors when making tar file
	    $tmpfile=`mktemp`; chomp ($tmpfile);
	    system ( "find $scen_bname/filesystems/$rootfs -type s > $tmpfile" );
	    #print ( "find $scen_bname/filesystems/$rootfs -type s > $tmpfile" );
	    $tar_opts="-X $tmpfile";
	    wlog (N, "--   Scenario excluded content:", '');
	    my $excl_content=`cat $tmpfile`;
	    wlog (N, "$excl_content", '');
	} else {
	    wlog (N, "--   rootfs not packaged (use option -r if you want to include it).", '');
	}
	
	
	# Estimating tar file size for progress bar 
	my $size=`du -sb --apparent-size $content | awk '{ total += \$1 - 512; }; END { print total }'`;
	wlog (V, "size=$size", '');
	if ( $inc_rootfs ) {
	    $size = int($size * 1040 / 1000);   
	} else {
	    $size = int($size * 1020 / 1000);
	}
	wlog (V, "size=$size", '');
	
	$tar_opts = "$tar_opts -C $cdir";
	
	print   "LANG=C tar -cpf - $content $tar_opts | pv -p -s ${size} | gzip > ${cdir}/${pack_tgz_name}.tgz\n";
	system ("LANG=C tar -cpf - $content $tar_opts | pv -p -s ${size} | gzip > ${cdir}/${pack_tgz_name}.tgz");
	wlog (N, "-- ...done", '');
	
	if ( defined($tmpfile) ) {
	    system ("rm $tmpfile")  
	}
		
}    

if ($opts{'unpack'}) {
    
    my $pack_file = $opts{'unpack'};
    vnx_die ("file $pack_file is not valid (perhaps does not exists).") unless ( -f $pack_file );
    wlog (N, "-- Unpackaging scenario $pack_file", '');

    print "LANG=C pv $pack_file | tar xzfp - \n";
    system ("LANG=C pv $pack_file | tar xzfp - ");
    
}    

 
            
sub wlog {

    my $log_level = shift; # Not used, included for compatibility with VNX wlog function
    my $msg = shift;
    my $logp = shift;

    print ("$logp $msg\n");
}

sub vnx_die {
   my $mess = shift;
   printf "$hline\n";
   printf "ERROR in %s:\n%s \n", (caller(0))[2], $mess;
   printf "$hline\n";
   exit 1;   
}